/* Copyright 2002 Scott Haywood

This file is part of the Label program.

Label is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Label is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Label; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <vector>
#include "chartable.h"
//#include "outline.h"

#ifndef NO_PS_GENERATOR
#include "putpostscript.h"
#endif

#undef SHOW
#define SHOW(X) cerr << #X "=" << (X) << endl
#undef SHOW2
#define SHOW2(X,Y) cerr << #X "=" << (X) << ", " #Y "=" << (Y) << endl

// adds the points representing a string to a vector
//
void Chartable::getPoints(const string & text,
               double x,
               double y,
               double wantedScaleX,
               double wantedScaleY,
               vector<vector< vector<Point> > > & vvvec) const
{
	if (text.size() < 1) return;
	const double sfx = scaleFactor(wantedScaleX);
	const double sfy = scaleFactor(wantedScaleY);
	double px, py;
	for (uint cx=0; cx<text.size(); cx++) {
		vector< vector<Point> > vv; // for this letter
		char c = text[cx];
		//SHOW(c); // ...........
		if (!contains(c)) // skip unknown chars
			continue;
    	const Charpath *ch = entry(c);
    	//SHOW(ch->npaths);
	    for (int i=0; i<ch->npaths; i++) {
    		vector<Point> v; // a vector for this path
			Path *gp = &ch->paths[i];
    		for (int j=0; j<gp->npoints; j++) {
    			px = gp->pts[j].x() * sfx + x;
    			py = gp->pts[j].y() * sfy + y;
    			v.push_back(Point(px, py));
    			//SHOW2(px,py); //..........
    		}
    		vv.push_back(v);
    	}
    	vvvec.push_back(vv);
		x += ch->final_moveto.x() * sfx;
	    y += ch->final_moveto.y() * sfy;
	}
}

#ifndef NO_PS_GENERATOR
void Chartable::postscript(const string & text,
                           double x,
                           double y,
                           double wantedScale,
                           ostream &os) const
{

	if (text.size() < 1) return;
	const double sf = scaleFactor(wantedScale);
	for (uint i=0; i<text.size(); i++) {
		char c = text[i];
		if (!contains(c))
			continue;
		postscript(c, x, y, wantedScale, os);
		const Charpath *cp = entry(c);
	    x += cp->final_moveto.x() * sf;
	    y += cp->final_moveto.y() * sf;
	    //
	}
}

void Chartable::postscript(char c,
                           double x,
                           double y,
                           double wantedScale,
                           ostream &os) const
{

	if (!contains(c))
		 return;
	const Charpath *cp = entry(c);
	if (cp->npaths < 1)
		return;
	const double sf = scaleFactor(wantedScale);
	ios::fmtflags oflags = os.flags(); // save flags ////
	os.precision(6);
	os.setf(ios::fixed);
	os.unsetf(ios::scientific);
	for (int i=0; i<cp->npaths; i++) {
		Path *gp = &cp->paths[i];
		putPostscript(gp->pts, gp->npoints, sf, x, y, os);
	}
    os << "closepath\n";
    os.flags(oflags); // restore flags ////
}
#endif
