/* Copyright 2002 Scott Haywood

This file is part of the Label program.

Label is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Label is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Label; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <vector>
#include "chartable2.h"
#include "vec.h"

#undef ENABLE_SHOW
#undef SHOW_NAME
#define SHOW_NAME "Chartable2"
#include "show.h"

Chartable2::StringPath::StringPath(
		Chartable2 *ct,
        const    string & text,
        double   wantedScaleX,  // horizontal scale factor
        double   wantedScaleY,  // vertical scale factor
        bool     closePaths,
        double   x,         // x translation
        double   y          // y translation
        )
{
	// Save input parameters in more useable form. 	
	uint nc = text.size();
	const double sfx = ct->scaleFactor(wantedScaleX);
	const double sfy = ct->scaleFactor(wantedScaleY);
	double translateX = x;
	double translateY = y;
	
	// create the array of Xforms
	xforms = new Xform[nc];
	
	// Traverse the input string
	for (uint ci=0; ci<nc; ci++) {  // for each character
		char c = text[ci];
		SHOW(c); // ...........
		if (!ct->contains(c)) { // unknown char
			cerr << "StringPath: unknown char 0x";
			cerr << hex << c << dec << endl;
			return; // ===================>>>>>>
		}
		
		xforms[ci] = Xform(sfx, sfy, translateX, translateY);
		
		const Chartable2::CharEntry & ce = *(ct->entry(c));
    	
		// save outlines for this char that are separate
		// from the fillable paths.
		//
		for (uint i=0; i < ce.n_separate_lines(); i++) {
			path_t & oline = createLinePath( ci );
			const Path & pth = ce.separate_line(i);
			SHOWF("sep lines, i=%d, path size = %d\n", i, pth.size());
			pth.toVec(oline);
		}

    	uint ng = ce.size();
    	SHOW(ng);
	    for (uint gi=0; gi<ng; gi++) {  // for each PathGroup
	    	const PathGroup & grp = ce[gi];
	    	
	    	// Create a new output fillable path,
	    	// and initialize a reference to it.
	    	path_t & op = createFillablePath( ci );
    		
    		uint npath = grp.size();
    		for (uint pathi=0; pathi<npath; pathi++) { // each path
    		
    			// Add this input path to the output
	    		//
	    		const Path &ipath = grp[pathi];
	    		uint npoint = ipath.size();
    			uint ifirst = op.size();
    			uint ilast = ifirst + npoint -1;
	    		ipath.toVec(op);
    			if ( npoint>1 && closePaths &&
					op[ifirst] != op[ilast] )
				{
					op.push_back( op.untransformed(ifirst) );
				}
#ifdef ENABLE_SHOW
				SHOWF("---\n");
				SHOW(op.size());
				for (uint i=0; i<op.size(); i++) {
					SHOW2(i,op[i]);
				}
				SHOWF("---\n");
#endif
    			if (ce.n_separate_lines() == 0) {
    				path_t & oline = createLinePath( ci );
    				ipath.toVec( oline );
    			}
    		}
    	}
    	translateX += ce.final_moveto.x() * sfx;
	    translateY += ce.final_moveto.y() * sfy;
	}
    SHOW2(line_paths.size(), fillable_paths.size());
    SHOWF("returning from StringPath,\n");
}

