/* Copyright 2002 Scott Haywood

This file is part of the Label program.

Label is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Label is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Label; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
 * Finds the point of intersection of
 * (1) a line thru a point at a given angle,
 * (2) a line determined by two points.
 * Assumes the two are not parallel.
 * 
 */

#include <iostream>
#include <string>
#include <cmath>
#include <cstdio>

#include "line_intersect.h"
#include "point.h"

#undef ENABLE_SHOW
#undef SHOW_NAME
#define SHOW_NAME "LINE_INTERSECT"
#include "show.h"

bool Util::nearlyVertical(double radians)
{
	double a = fabs(radians);
	double diff = fabs(M_PI_2 - a);
	const double tolerance = 0.00001;
	if (diff < tolerance)
		return true;
	diff = fabs(3 * M_PI_2 - a);
	return (diff < tolerance);
	
}

/*
 * Find the intersection of a line thru a given
 * point with a line determined by two points.
 */
Point Util::findIntersection(Point p,
                       double radians,
                       Point a,
                       Point b)
{
	double px = p.x();
	double py = p.y();
	//SHOW2(px,py);
	double ax = a.x();
	double ay = a.y();
	//SHOW2(ax,ay);
	double bx = b.x();
	double by = b.y();
	//SHOW2(bx,by);

	double x,y;
	double m2; // slope of the segment from a to b
	bool lineSegVert = false;
	
	if (ax == bx)
		lineSegVert = true;
	else {
		m2 = (by-ay)/(bx-ax);
		//             1759218604441599.5
		if (fabs(m2) > 100000000000000.0)
			lineSegVert = true; // nearly vertical
	}
	if (lineSegVert) {
		//SHOWF("line segment vertical or nearly vertical\n");
		if (nearlyVertical(radians)) {
			//SHOWF("both lines vertical or nearly vertical\n");	
		}
		double m1 = tan(radians);
		double c1 = py - m1 * px;
		x = ax;
		y = m1 * x + c1;
	}
	else {
		double c2 = by - m2 * bx;
		//SHOWF("m2=%.18f, c2=%.18f\n", m2, c2);
		if (nearlyVertical(radians)) {
			//SHOWF("nearlyVertical(%.18f) is true\n", radians);
			x = px;
			y = m2 * x + c2;
		}
		else {
			double m1 = tan(radians);
			double c1 = py - m1 * px;
			if (fabs(m1-m2) < 0.000001) {
				//SHOWF("slopes equal (parallel lines)\n");
			}
			//SHOWF("m1=%.18f, m2=%.18f\n", m1, m2);
			x = (c1-c2)/(m2 - m1);
			y = m2 * x + c2;
			//SHOW2(x,y);
		}
	}
	//SHOWF("=================\n");
	//SHOWF("A line thru (%.18f,%.18f),\n", px, py);
	//SHOWF("at angle %.18f (%.18f degrees),\n", radians, radians * 180.0 / M_PI);
	//SHOWF("intersects the line determined by\n");
	//SHOWF("points (%f,%f) and (%f,%f)\n", ax,ay,bx,by);
	//SHOWF("at point (%.18f, %.18f)\n", x,y);
	//SHOWF("=================\n");
	return Point(x,y);
}

