/* Copyright 2002 Scott Haywood

This file is part of the Label program.

Label is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Label is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Label; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef LINE_INTERSECT_INCLUDED
#define LINE_INTERSECT_INCLUDED

/****************************************************************
**
** Declaration of functions in the Util namespace
** that perform geometry calculations
**
****************************************************************/

#include "point.h"

namespace Util {

// Returns true if the given angle (in radians) is approximately
// equal to 90 degrees or 270 degrees.
//
bool nearlyVertical(double radians);

/*
 * Finds the intersection of a line thru a given
 * point, p, with a line determined by two points, a and b.
 */
Point findIntersection(Point p,
                       double radians,
                       Point a,
                       Point b);

inline bool findSegmentIntersection(
	Point p,
	double radians,
    Point a,
    Point b,
    Point * out)
{
	Point x = findIntersection(p, radians, a, b);
	if ( ( x.x() < a.x() && x.x() < b.x() ) ||
	     ( x.x() > a.x() && x.x() > b.x() ) ||
         ( x.y() < a.y() && x.y() < b.y() ) ||
         ( x.y() > a.y() && x.y() > b.y() )
    ) {
		return false;
	} else {
		*out = x;
		return true;
	}
}


}

#endif
