/* Copyright 2002 Scott Haywood

This file is part of the Label program.

Label is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Label is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Label; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef INCLUDED_POINT
#define INCLUDED_POINT

#include <iostream>
#include <cmath>

/*
 *	Point represents a point in 2-dimensional space,
 *	with coordinate values x and y.
 */

class Point {
//public:
	double xval;
	double yval;
public:
	void setX(double v) { xval = v; }
	void setY(double v) { yval = v; }
	double x() const { return xval; }
	double y() const { return yval; }

	// Returns the distance between this and a given point
	double distance(const Point& p) const {
		return hypot(xval - p.xval, yval - p.yval);
	}

	// Returns a point on the line segment defined by
	// this and another point.  The distance from this point
	// to the returned point will be the segment length times
	// the given ratio.
	Point toward(const Point& p, double ratio) {
		return Point( xval + (p.xval - xval) * ratio,
                      yval + (p.yval - yval) * ratio );
    }

	// Returns the midpoint of a line segment defined by
	// this and a given point.
    Point midPoint(const Point& p) {
    	return toward(p, 0.5);
    }

	Point() : xval(0), yval(0) {}
	Point(double a, double b) : xval(a), yval(b) {}

	bool operator==(const Point & b) const {
		return (xval == b.xval) && (yval == b.yval);
	}
	bool operator<(const Point & b) const {
		return (xval < b.xval) ? true :
		 (((xval == b.xval) && (yval < b.yval)) ?  true : false);
	}
	Point operator-() {
		return Point(-xval, -yval);
	}
	Point operator+(const Point &b) {
		return Point(xval + b.xval, yval + b.yval);
	}
	Point operator-(const Point &b) {
		return Point(xval - b.xval, yval - b.yval);
	}
	//Point transform(double mx, double my, double tx, double ty) {
	//	return Point(xval * mx + tx, yval * my + ty);
	//}
	friend ostream & operator<<(ostream & os, const Point &p) {
		os << p.x() << "," << p.y();
		return os;	
	}
};

#endif

