/* Copyright 2002 Scott Haywood

This file is part of the Label program.

Label is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Label is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Label; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
** Macros for debugging
** To use them,
**	#undef SHOW_NAME                  // |
**	#define SHOW_NAME "<Module name>" // | optional
**	#define ENABLE_SHOW
**	#include "show.h"
**
** Then, for example,
**	int x = some_function();
**	SHOW(x);
**	SHOW(another_function(x));
**
** If SHOWF is used, you must also include
** a declaration of printf
**	#include <cstdio>
*/

// This file can be processed multiple times.
// For example if both x.h and x.cpp include it,
// and x.cpp includes x.h,
// one can have ENABLE_SHOW defined, and the other
// have it undefined.
//#ifndef SHOW_MACROS_INCLUDED
//#define SHOW_MACROS_INCLUDED

#include <iostream>

#ifndef SHOW_NAME
#define SHOW_NAME "debug"
#endif

#undef SHOW0
#undef SHOW
#undef SHOW2
#undef SHOW_NMVAL
#undef SHOWF

#ifdef ENABLE_SHOW

#define SHOW0 cout << SHOW_NAME << ": "
#define SHOW(X) SHOW0 << #X "=" << (X) << endl
#define SHOW2(X,Y) SHOW0 << #X "=" << (X) << ", " << #Y "=" << (Y) <<endl
#define SHOW_NMVAL(X) #X "=" << (X)
#define SHOWF SHOW0; printf

#else

#define SHOW0 if(false) cout << SHOW_NAME << ": "
#define SHOW(X)
#define SHOW2(X,Y)
#define SHOW_NMVAL(X)
#define SHOWF if (false) printf

#endif

//#endif


