/* Copyright 2002 Scott Haywood

This file is part of the Label program.

Label is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Label is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Label; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef UTIL_VEC_H
#define UTIL_VEC_H

/*
**	Vec<T> is a vector<T>.  The operator[] does
**	range-checking; i.e. it checks its numeric parameter
**	and throws out_of_range if the number is greater than
**	or equal to the size of the vector.
**/


#include <vector>
#include <string>
#include <stdexcept>
#include <cstdio>

namespace Util {

template <class T>
class Vec : public vector<T>
{
	void check(size_type i) const {
		if (i >= size()) {
			char buf[64];
			sprintf(buf, "out of range: Vec: at(%Zu), size=%Zu", i, size());
			throw out_of_range(string(buf));
		}
	}
public:
	Vec() : vector<T>() {}
	Vec(size_type n) : vector<T>(n) {}
	T& at(size_type i)             {check(i); return *(begin() + i);}
	const T& at(size_type i) const {check(i); return *(begin() + i);}
	T& operator[](size_type i)             { return at(i); }
	const T& operator[](size_type i) const { return at(i); }
};

};

#endif
