/* Copyright 2002 Scott Haywood

This file is part of the Label program.

Label is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Label is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Label; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/****************************************************************************
** 
** Implementation of ConfigDialog,
** which provides a user interface for
** configuring some parts of the drawn image.
**
**
****************************************************************************/
#include "config_dialog.h"
#include "factory.h"

#include <qvariant.h>
#include <qbuttongroup.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qlineedit.h>

#undef ENABLE_SHOW
#undef SHOW_NAME
#define SHOW_NAME "ConfigDialog"
#include "show.h"

/* 
 *  Constructs a ConfigDialog which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f'.
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
ConfigDialog::ConfigDialog( QWidget* parent,
                            const char* name,
							bool modal,
							unsigned int /* WFlags */ fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
		setName( "ConfigDialog" );
    //resize( 500, 480 );
    setCaption( trUtf8( "Configure" ) );

    QWidget* privateLayoutWidget = new QWidget( this, "VertWidget" );
    //privateLayoutWidget->setGeometry( QRect( 20, 20, 500, 180 ) );
    VertLayout = new QVBoxLayout( privateLayoutWidget, 0, 6, "VertLayout");

    ButtonGroup1 = new QButtonGroup( privateLayoutWidget, "ButtonGroup1" );
    ButtonGroup1->setTitle( trUtf8( "" ) );
    ButtonGroup1->setColumnLayout(0, Qt::Vertical );
    ButtonGroup1->layout()->setSpacing( 6 );
    ButtonGroup1->layout()->setMargin( 11 );
    ButtonGroup1Layout = new QHBoxLayout( ButtonGroup1->layout() );
    ButtonGroup1Layout->setAlignment( Qt::AlignTop );

    RadioButtonRandom = new QRadioButton( ButtonGroup1, "RB_Random" );
    RadioButtonRandom->setText( trUtf8( "Random" ) );
    ButtonGroup1Layout->addWidget( RadioButtonRandom );

    RadioButtonRectangle = new QRadioButton( ButtonGroup1, "RB_Rectangle" );
    RadioButtonRectangle->setText( trUtf8( "Rectangle" ) );
    ButtonGroup1Layout->addWidget( RadioButtonRectangle );

    RadioButtonOval = new QRadioButton( ButtonGroup1, "RB_Oval" );
    RadioButtonOval->setText( trUtf8( "Ovoid" ) );
    ButtonGroup1Layout->addWidget( RadioButtonOval );
    VertLayout->addWidget( ButtonGroup1 );
    connect( ButtonGroup1, SIGNAL(clicked(int)),
	         this,         SLOT(setType()) );
	
	// calculate the geometry (there must be a better way)
	int w1, h1;
	w1 = RadioButtonRandom->geometry().width() +
              RadioButtonRectangle->geometry().width() +
              RadioButtonOval->geometry().width() +
              ButtonGroup1Layout->spacing() * 2 +
              ButtonGroup1Layout->margin() * 2; //////////
    h1 = ButtonGroup1->geometry().height(); //////////
    SHOW2(w1,h1);

    Layout5 = new QHBoxLayout( 0, 0, 6, "Layout5");
    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    Layout5->addItem( spacer );

    TextLabel2 = new QLabel( privateLayoutWidget, "TextLabel2" );
    TextLabel2->setText( trUtf8( "Number of Points" ) );
    Layout5->addWidget( TextLabel2 );

    SpinBox1 = new QSpinBox( privateLayoutWidget, "SpinBox1" );
    SpinBox1->setMinValue(2);
    SpinBox1->setMaxValue(500);
    Layout5->addWidget( SpinBox1 );
    QSpacerItem* spacer_2 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    Layout5->addItem( spacer_2 );
    VertLayout->addLayout( Layout5 );

    int w2, h2;
    w2 = TextLabel2->geometry().width() +
    	SpinBox1->geometry().width() +
    	Layout5->spacing() +
    	Layout5->margin() * 2;
    h2 = max(TextLabel2->geometry().height(),
    	          SpinBox1->geometry().height() );
    SHOW2(w2,h2);

 	QHBoxLayout * Layout7;
 	Layout7 = new QHBoxLayout( 0, 0, 6, "Layout7");
 	QLabel * TextLabel3;
 	TextLabel3 = new QLabel( privateLayoutWidget, "TextLabel3" );
    TextLabel3->setText( trUtf8( "Label text: " ) );
    Layout7->addWidget( TextLabel3 );
    LineEdit1 = new QLineEdit( privateLayoutWidget, "LineEdit1" );
    Layout7->addWidget( LineEdit1 );
    VertLayout->addLayout( Layout7 );

    int w4, h4;
    w4 = TextLabel3->geometry().width() +
    	 LineEdit1->geometry().width() +
    	 Layout7->spacing(); 
    h4 = max(TextLabel3->geometry().height(),
    	     LineEdit1->geometry().height() );
    SHOW2(w4,h4);

    Layout6 = new QHBoxLayout( 0, 0, 6, "Layout6");
	ButtonOk = new QPushButton(privateLayoutWidget, "ButtonOk");
	ButtonOk->setText( trUtf8( "Ok" ) );
	Layout6->addWidget( ButtonOk );
    connect( ButtonOk, SIGNAL(clicked()),
	         this,       SLOT(accept()) );
	
    QSpacerItem* spacer_3 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	Layout6->addItem( spacer_3 );
	
	ButtonCancel = new QPushButton(privateLayoutWidget, "ButtonCancel");
	ButtonCancel->setText( trUtf8( "Cancel" ) );
	Layout6->addWidget( ButtonCancel );
	connect( ButtonCancel, SIGNAL(clicked()),
	         this,           SLOT(reject()) );
    VertLayout->addLayout( Layout6 );

    int w3, h3;
    w3 = ButtonOk->geometry().width() +
    	 ButtonCancel->geometry().width() +
    	 Layout6->spacing();
    h3 = max(ButtonOk->geometry().height(),
    	     ButtonCancel->geometry().height() );
    SHOW2(w3,h3);

	int ww = max(max(max(w1,w2),w3),w4) + 2 * VertLayout->margin();
 	int hh = h1 + h2 + h3 + h4 +
 	         2 * VertLayout->spacing() + 2 * VertLayout->margin();
    SHOW2(ww, hh);
    //privateLayoutWidget->setGeometry( QRect( 20, 20, ww, hh ) );
    privateLayoutWidget->setMinimumSize(ww, hh);
    setMinimumSize(ww,hh); // prevent frame from being made too small

    // set default values
    setNPoints(3);
    setType(Factory::Random);
	enableNPoints();
    setLabelText("Nice Label");
}

/*  
 *  Destroys the object and frees any allocated resources
 */
ConfigDialog::~ConfigDialog()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 * Sets the type property, which is the type of polygon
 * to be created, and modifies the user interface accordingly.
 * The types are defined in the Factory class.
 */
void ConfigDialog::setType(int t) {
	type = t;
	switch(t) {
	case Factory::Random :
		RadioButtonRandom->setChecked(true);
		enableNPoints();
		break;
	case Factory::Rectangle :
		RadioButtonRectangle->setChecked(true);
		disableNPoints();
		break;
	case Factory::Oval :
		RadioButtonOval->setChecked(true);
		enableNPoints();
		break;
	}
}

void ConfigDialog::setNPoints(int n) {
	SpinBox1->setValue(n);
}

// setType() without parameters is invoked in response
// to a click of one of the radio buttons.
void ConfigDialog::setType() {
	if (RadioButtonRandom->isChecked()) {
		setType(Factory::Random);
	}
	else if (RadioButtonRectangle->isChecked()) {
		setType(Factory::Rectangle);
	}
	else if (RadioButtonOval->isChecked()) {
		setType(Factory::Oval);
	}
}

void ConfigDialog::setLabelText(const QString & s)
{
	LineEdit1->setText(s);
}

