/* Copyright 2002 Scott Haywood

This file is part of the Label program.

Label is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Label is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Label; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef LABEL_CONFIG_DIALOG_H
#define LABEL_CONFIG_DIALOG_H

/****************************************************************
**  ConfigDialog allows the user to configure the application.
**
****************************************************************/


#include <qdialog.h>
#include <qnamespace.h>
#include <qbuttongroup.h>
#include <qlayout.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qpushbutton.h>
#include <qlineedit.h>

/*
 *	ConfigDialog allows the user to configure the application.
 *
 *  When the user requests to configure,
 *  the program a creates ConfigDialog, sets its properties
 *	(by calling setType(int), etc.) to match the current state
 *	of the drawing, and displays the dialog.  If the dialog
 *	sends an accept signal, the program reads the dialog
 *	properties and changes the drawing accordingly.
 */
class ConfigDialog : public QDialog
{
Q_OBJECT

int type;
int nPoints;
QString labelText;

QButtonGroup * ButtonGroup1;
QVBoxLayout * VertLayout;
QHBoxLayout * ButtonGroup1Layout;
QRadioButton * RadioButtonRandom;
QRadioButton * RadioButtonRectangle;
QRadioButton * RadioButtonOval;
QHBoxLayout * Layout5;
QLabel * TextLabel2;
QSpinBox * SpinBox1;
QPushButton * ButtonOk;
QPushButton * ButtonCancel;
QHBoxLayout * Layout6;
QLineEdit * LineEdit1;

public:

/* 
 *  Constructs a ConfigDialog which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f'.
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
ConfigDialog( QWidget* parent,
              const char* name,
              bool modal,
              unsigned int /* WFlags */ fl = 0);
~ConfigDialog();

// Returns the polygon type.
int getType() const { return type; }

// Returns the number of points the polygon should have.
int getNPoints() const { return SpinBox1->value(); }

// Returns the text to be used for the label.
QString getLabelText() { return LineEdit1->text(); }

public slots:

/*
 * Sets the type property, which is the type of polygon
 * to be created, and modifies the user interface accordingly.
 * The types are defined in the Factory class.
 */
void setType(int t);

/*
 * Sets the number of points to be used in creating the polygon.
 */
void setNPoints(int n);

/*
 * Sets the text to be used for the label.
 */
void setLabelText(const QString & s);

private slots:

//
void setType();

// Allows the user to change the number of points
void enableNPoints()  { SpinBox1->setEnabled(true); }

// Prevents the user from changing the number of points
void disableNPoints() { SpinBox1->setEnabled(false); }

};

#endif LABEL_CONFIG_DIALOG_H
