/* Copyright 2002 Scott Haywood

This file is part of the Label program.

Label is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Label is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Label; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <qwmatrix.h>
#include "drawing_view.h"

#undef ENABLE_SHOW
#undef SHOW_NAME
#define SHOW_NAME "DrawingView"
#include "show.h"

/*
 * Reimplementation of virtual methods of QScrollView
 */
void DrawingView::contentsMousePressEvent ( QMouseEvent * qme)
{
	// Find the first item marked "draggable" that is
	// at the mouse position.
	QPoint p;
	QWMatrix mat = inverseWorldMatrix();
	//p = inverseWorldMatrix().map(qme->pos()); // compiler bug
	p = mat.map(qme->pos());
    QCanvasItemList l=canvas()->collisions(p);
    for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it) {
		if ( !isDraggable(*it) ) {
	    	continue;
		}
		moving = *it;
		moving_start = p;
		return;
    }
    moving = 0;
}

void DrawingView::contentsMouseMoveEvent ( QMouseEvent * qme)
{
#ifdef ENABLE_SHOW
	QPoint g = qme->globalPos();
	SHOWF("Move: \n");
	SHOW2(g.x(), g.y());
#endif
    if ( moving ) {
		QPoint p = inverseWorldMatrix().map(qme->pos());
		moving->moveBy(p.x() - moving_start.x(),
		               p.y() - moving_start.y());
		moving_start = p;
		canvas()->update();
    }
}

void DrawingView::contentsMouseReleaseEvent ( QMouseEvent * )
{
	SHOWF("DrawingView::contentsMouseReleaseEvent\n");
#ifdef ENABLE_SHOW
	QPoint g = qme->globalPos();
	SHOWF("Release: \n");
	SHOW2(g.x(), g.y());
#endif
	emit released(moving);
}
