/* Copyright 2002 Scott Haywood

This file is part of the Label program.

Label is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Label is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Label; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef LABEL_DrawingView_H
#define LABEL_DrawingView_H

/*
 * DrawingView is a type of QCanvasView
 * that allows some items to be dragged to
 * a new location
 */

#include <map>
#include <qcanvas.h>

class DrawingView : public QCanvasView {

	Q_OBJECT

private:
	map<QCanvasItem*,bool> draggable_map;
	QCanvasItem * moving;
	QPoint moving_start;
	
protected:
	// reimplemented from QScrollView
	void contentsMousePressEvent ( QMouseEvent * e);		
    void contentsMouseMoveEvent ( QMouseEvent * e);
    void contentsMouseReleaseEvent ( QMouseEvent * e);

public:
	//DrawingView ( QWidget * parent = 0, const char * name = 0, WFlags f = 0 )
	//: QCanvasView ( parent, name, f ) { }

	DrawingView ( QCanvas * canvas, 
	              QWidget * parent = 0, const char * name = 0, WFlags f = 0 )
	: QCanvasView ( canvas, parent, name, f ) { }


	/*
	*	Marks a canvas item as either draggable (true)
	*	or not (false).
	*/
	void setDraggable(QCanvasItem *ip, bool draggable) {
		draggable_map[ip] = draggable;
	}

	/*
	*	Marks a canvas item as not draggable.
	*/
	void removeFromDraggable(QCanvasItem *ip) {
		draggable_map.erase(ip);
	}

	/*
	*	Returns true iff a canvas item is draggable.
	*/
	bool isDraggable(QCanvasItem *ip) {
		return draggable_map[ip];
	}
signals:

	/*
	*	`released' is a Qt signal which is emitted
	*	when the mouse button is released after
	*	dragging a canvas item.
	*/
	void released(QCanvasItem *ip);

};


#endif LABEL_DRAWING_VIEW_H
