/* Copyright 2002 Scott Haywood

This file is part of the Label program.

Label is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Label is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Label; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <cmath>

#include "factory.h"
#include "rand.h"
#include "badnum.h"
#include "bounds.h"

#undef ENABLE_SHOW
#undef SHOW_NAME
#define SHOW_NAME "Factory"
#include "show.h"

/*
 *	Store an sequence of random points
 *	into the given vector.
 */
void Factory::getRandomPoints(vector<Point> & v) const
{
	double ww = 0;
	double hh = 0;
	const QRect & b = bounds;
	do {
		v.clear();
		for(int i=0; i<nPoints; i++) {
			int x = Util::myRand( b.x(), b.x()+b.width() );
			int y = Util::myRand( b.y(), b.y()+b.height() );
			if (x < b.x() || y < b.y() ||
			    x > b.x()+b.width() || y > b.y()+b.height())
			{
				SHOW2(x, b.x());
				SHOW2(y, b.y());
				throw "x,y  out of range";
			}
			v.push_back(Point(x,y));
		}
		Util::getBounds(v,0,0,&ww,&hh);
#if 0
		SHOWF("====\n");
		SHOW2(ww,hh);
		SHOW2(nPoints, v.size());
		for(int i=0; i<nPoints; i++) {
			SHOW(v[i]);
		}
		SHOWF("====\n");
#endif
	} while	(ww == 0 || hh == 0);
}

/*
 *	Create an instance of Polygon.
 */
Polygon * Factory::createPolygon() const
{
    Polygon * p = 0;
	vector<Point> pa;
	const QRect & b = bounds;

	SHOW2(type,nPoints);
	SHOW2(b.x(), b.y());
	SHOW2(b.width(), b.height());			
	switch(type) {
  	case Special : // for debug
  		pa.push_back( Point(151,181) );
		pa.push_back( Point(153,140) );
		break;
  	case Random :
  		getRandomPoints(pa);
		break;
  	case Oval :
		//throw ("Oval not implemented");
		{
		 double r_step = (M_PI * 2) / nPoints;
		 double r_lim = r_step * nPoints;
		 //double cx = b.x() + b.width()/2.0;
		 //double cy = b.y() + b.height()/2.0;
		 double radius = max(b.width(), b.height()) / 2.0;
		 for (double angle = r_step/2.0; angle <= r_lim; angle += r_step) {
			double x = radius * cos(angle);
			double y = radius * sin(angle);
			pa.push_back( Point(x,y) );
			SHOW2(x,y);
		 }
		}
		break;
  	case Rectangle :
		//throw ("Rectangle not implemented");
		
		// ignores nPoints
		pa.push_back( Point(b.x(), b.y()) );
		pa.push_back( Point(b.x() + b.width(), b.y()) );
		pa.push_back( Point(b.x() + b.width(), b.y() + b.height()) );
		pa.push_back( Point(b.x(), b.y() + b.height()) );
		break;
	}
	p = new Polygon(canvasp, pa, bounds);
	
	///////////
#if 0
  {
	Outline * o1 = p->getOutline();
	vector<Point> v;
	o1->getPoints(v);
	bool bad = false;
	for (unsigned int i=0; i<v.size(); i++) {
		Point a = v[i];
		//SHOW2(i,a);
		if (Util::badNum(a.x()) || Util::badNum(a.y())) {
			bad = true;
			break;
		}
	}
	//SHOW(bad);
	if (bad) {
		SHOWF("----\n");
		SHOWF("Bad outline --\n");
		SHOWF("points:\n");
		for (unsigned int i=0; i<pa.size(); i++) {
			SHOW2(i, pa[i]);
		}
		SHOWF("outline:\n");
		for (unsigned int i=0; i<v.size(); i++) {
			SHOW2(i, v[i]);
			//SHOW2( badNum(v[i].x()), badNum(v[i].y()) );
		}
		SHOWF("----\n");
	}
  }
#endif
    ///////////
	return p;
}
