/* Copyright 2002 Scott Haywood

This file is part of the Label program.

Label is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Label is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Label; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef LABEL_FACTORY_H
#define LABEL_FACTORY_H

/****************************************************************
**
**	Factory creates instances of Polygon.
**
**
****************************************************************/

#include <iostream>
#include <vector>
#include <qcanvas.h>
#include "polygon.h"
#include "point.h"

class Factory
{
int type;
int nPoints;
QRect bounds;
QCanvas *canvasp;
void getRandomPoints(vector<Point> &v) const;

public:

// The polygon types.  Special is used for debugging.
enum Type { Random, Rectangle, Oval, Special };

// Creates a Factory
Factory() : type(Random), nPoints(3), bounds(QRect()), canvasp(0) {}
~Factory() {}

/*
 * Sets the type of polygon to be created;
 * should be one of the Type enum.
 */
void setType(int t) { type = t; }

/*
 * Gets the type of polygon that the Factory will create.
 */
int getType() const { return type; }

/*
 * Sets the number of points that the polygon will have,
 * if type is not Rectangle.
 */
void setNPoints(int n) { nPoints = n; }

/*
 * Gets the "number of points" property.
 * See setNPoints. 
 */
int getNPoints() const { return nPoints; }

/*
 * Sets the rectangular bounds into which the polygon
 * should precisely fit.
 */
void setBounds(const QRect & r) { bounds = r; }

/*
 * Gets the rectangular bounds used for scaling the polygon.
 * See setBounds. 
 */
QRect getBounds() const { return bounds; }

/*
 * Sets the canvas to which the polygon should be attached.
 */
void setCanvas(QCanvas * cp) { canvasp = cp; }

/*
 * Gets the canvas to which the polygon will be attached.
 */
QCanvas * getCanvas() { return canvasp; }

/*
 * Creates a Polygon
 */
Polygon * createPolygon() const;

};

#endif LABEL_FACTORY_H
