/* Copyright 2002 Scott Haywood

This file is part of the Label program.

Label is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Label is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Label; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/****************************************************************
**
** Draw a labeled shape, using Qt
**
****************************************************************/
#include <iostream>

#include <qapplication.h>
#include <qpushbutton.h>
#include <qlcdnumber.h>
#include <qfont.h>
#include <qlayout.h>
#include <qaction.h>
#include <qtooltip.h>
#include <qvbox.h>

#include "labeldrawing.h"
#include "config_dialog.h"

// for getopt()
#include <unistd.h>

#undef ENABLE_SHOW
#include "show.h"


/*
 *	MyWidget contains a LabelDrawing and controls for it.
 */
#undef SHOW_NAME
#define SHOW_NAME "MyWidget"

class MyWidget: public QWidget
{
	//QAction configAction;
	//QAction quitAction;
	//QAction newAction;
	//QAction nextAction;
	
public:
    MyWidget( QWidget *parent=0, const char *name=0 );
    ~MyWidget() {
    	SHOW0 << "Enter ~MyWidget\n";
    	//delete configAction;
    	//delete quitAction;
    	//delete newAction;
    	//delete nextAction;
    	SHOW0 << "exit ~MyWidget\n";
    }
    void init();
    void polish(); // virtual
    void showEvent(QShowEvent *); // virtual
};

void MyWidget::showEvent(QShowEvent *e)
{
	QSize wsize = size();
    //cerr << "showEvent: size = " << wsize.width() << "," << wsize.height() << endl;

    // set the widget's location on the screen
    //w.setGeometry( 100, 100, wsize.width(), wsize.height() );
    QDesktopWidget d;
    QSize ssize = d.screen()->size();
    move( (ssize.width() - wsize.width())/2,
            (ssize.height() - wsize.height())/2 );

    QWidget::showEvent(e);
}

void MyWidget::polish()
{
	QWidget::polish();
	//QSize wsize = size();
    //cerr << "polish: size = " << wsize.width() << "," << wsize.height() << endl;
}

MyWidget::MyWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
	init();
}

void MyWidget::init()
{
    // create the drawing
    QSize ssize = QDesktopWidget().screen()->size();
    LabelDrawing *labelDrawing 	=
    	new LabelDrawing( ssize.width()/2,
    	                  ssize.height()/2,
    	                  this, "labelDrawing" );
	QToolTip::add(labelDrawing,
		trUtf8("the label can be dragged to a different position"));

    // create buttons and actions

    QVBox * vbox1 = new QVBox(this, "vbox1");

    QPushButton *nextButton = new QPushButton( "Next", vbox1, "next" );
    nextButton->setFont( QFont( "Times", 18, QFont::Bold ) );
    connect( nextButton, SIGNAL(clicked()),
	         labelDrawing, SLOT(next()) );
    QAction *nextAction = new QAction( this, "nextAction");
    nextAction->setMenuText( trUtf8("Next") );
    connect( nextAction, SIGNAL(activated()),
	         nextButton, SLOT(animateClick()) );
	QToolTip::add(nextButton, trUtf8("move the label to next position"));

    QPushButton *prevButton = new QPushButton( "Previous", vbox1, "prev" );
    prevButton->setFont( QFont( "Times", 18, QFont::Bold ) );
    connect( prevButton, SIGNAL(clicked()),
	         labelDrawing, SLOT(prev()) );
    QAction *prevAction = new QAction( this, "prevAction");
    nextAction->setMenuText( trUtf8("Previous") );
    connect( prevAction, SIGNAL(activated()),
	         prevButton, SLOT(animateClick()) );
	QToolTip::add(prevButton, trUtf8("move the label to previous position"));

    QPushButton *newPoly = new QPushButton( "New", this, "new" );
    newPoly->setFont( QFont( "Times", 18, QFont::Bold ) );
    connect( newPoly, SIGNAL(clicked()),
	         labelDrawing, SLOT(newPolygon()) );
	QToolTip::add(newPoly, trUtf8("create a new figure to be labelled"));

    QPushButton *quitButton = new QPushButton( "Quit", this, "quit" );
    quitButton->setFont( QFont( "Times", 18, QFont::Bold ) );
    // note: qApp is the global application object
    connect( quitButton, SIGNAL(clicked()), qApp, SLOT(quit()) );
    QAction *quitAction = new QAction( trUtf8("Quit"), trUtf8("quit"),
    	CTRL+Key_S, this, "quitAction");
    connect( quitAction, SIGNAL(activated()),
	         quitButton, SLOT(animateClick()) );
	QToolTip::add(quitButton, trUtf8("quit the application"));

    QPushButton *cfgButton = new QPushButton( "Configure", this, "config" );
    cfgButton->setFont( QFont( "Times", 18, QFont::Bold ) );
    connect( cfgButton, SIGNAL(clicked()),
             labelDrawing, SLOT(configure()) );
	QToolTip::add(cfgButton, trUtf8("configure the drawing"));

    QGridLayout *grid = new QGridLayout( this, 7, 2, 10 );
    //3x2, 10 pixel border

    grid->addWidget( quitButton, 0, 0 );
    QSpacerItem* spacer = new QSpacerItem(
    	20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    grid->addItem(       spacer, 1, 0 );

    //grid->addWidget( nextButton, 2, 0 );
    grid->addWidget(vbox1, 2, 0);

    QSpacerItem* spacer2 = new QSpacerItem(
    	20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    grid->addItem(      spacer2, 3, 0 );
    grid->addWidget(    newPoly, 4, 0 );
    QSpacerItem* spacer3 = new QSpacerItem(
    	20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    grid->addItem(      spacer3, 5, 0 );
    grid->addWidget(  cfgButton, 6, 0 );
    grid->addMultiCellWidget( labelDrawing, 0, 6, 1, 1 );
    //grid->setColStretch( 1, 10 );

    nextButton->setFocus();

    //grid->activate(); // useless here
}

#undef SHOW_NAME
#define SHOW_NAME "main"

char *argv0;

void usage() {
	cerr << "usage:\n";
	cerr << "   " << argv0 << " [ -r ]\n";
	cerr << "Options:\n";
	cerr << "   -r\tSeed the random number generator\n";
}
	
int main( int argc, char **argv )
{
    argv0 = argv[0];

	// Parse the command-line options.
	//
	int c;
	while ((c = getopt(argc, argv, "r")) !=EOF) {

		switch (c) {
		case 'r' :
			srand(time(0));
			break;
		default :
			usage(); exit(-1);
		}
	}

 try {
    QApplication a( argc, argv );

    MyWidget w;
    QSize wsize = w.size();
    //cerr << "w size = " << wsize.width() << "," << wsize.height() << endl;

    a.setMainWidget( &w );
    //wsize = w.size();
    //cerr << "w size (after a.setMainWidget(&w))= "
    //	<< wsize.width() << "," << wsize.height() << endl;
    	
    w.show();

    wsize = w.size();
    //cerr << "w size (after show)= "
    //	<< wsize.width() << "," << wsize.height() << endl;

    int rv = a.exec();
    //cerr << "main done\n";
    return rv;
 }
 catch (char *p) { cerr << "caught: " << p << endl; throw; }
}
