/* Copyright 2002 Scott Haywood

This file is part of the Label program.

Label is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Label is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Label; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef LABEL_POLYGON2_H
#define LABEL_POLYGON2_H

/****************************************************************
**
**	Polygon is a QCanvasPolygon constructed
**	from a given sequence of points.
**
****************************************************************/

#include <iostream>
#include <vector>

#include <qcanvas.h>
#include "point.h"
#include "outline.h"
#include "outlined_polygon.h"

class Polygon : public OutlinedPolygon
{
public:

	// Constructs a Polygon from the vector pa,
	// with the points transformed to fit within the
	// rectangle bounds.
	//
	Polygon(QCanvas *cp,
	        const vector<Point> & pa,
	        const QRect & bounds);

	~Polygon();

	// Reimplementation of the Qt method (QCanvasPolygonalItem::drawShape)
	void drawShape(QPainter & p);
	
	// The drawn_filled property determines whether the shape
	// is drawn as a filled solid.
	void setDrawnFilled(bool x) { drawn_filled = x; }
	bool drawnFilled() { return drawn_filled; }
	
	// The outline_drawn property determines whether the outline is drawn.
	void setOutlineDrawn(bool x) { outline_drawn = x; }
	bool outlineDrawn() { return outline_drawn; }
	
private:
	// don't allow copy constructor or assignment.
	// This is to make it easier to manage pointers
	// that are deleted by the destructor.
	//
	Polygon(const Polygon &r) :
		OutlinedPolygon(r.canvas())
		{ throw "Polygon: no copy constructor\n"; }
	Polygon& operator=(const Polygon &)
		{ throw "Polygon: no assignment operator\n"; }
	
	QPointArray inner_path;
	QPointArray outer_path;
	bool drawn_filled;
	bool outline_drawn;
};

#endif LABEL_POLYGON_H
