/* Copyright 2002 Scott Haywood

This file is part of the Label program.

Label is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Label is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Label; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/****************************************************************
**
** Implementation of TextPolygon
**
****************************************************************/

#include <iostream>
#include <vector>
#include <cassert>
//#include <qfont.h>

#include "text.h"
#include "outline.h"
#include "chartable.h"
#include "point.h"

#ifdef USE_CHARTABLE2
#include "chartable2.h"
#else
#include "chartable.h"
#endif

#undef ENABLE_SHOW
#undef SHOW_NAME
#define SHOW_NAME "TextPolygon"
#include "show.h"

#ifdef USE_CHARTABLE2
extern Chartable2 chartable2;
#else
extern Chartable chartable;
#endif


TextPolygon::TextPolygon(QCanvas *cp, const QString & s, const QRect & bounds)
	: OutlinedPolygon(cp), text(s)
{
	//const QRect & b = bounds;
	
	//qDebug("TextPolygon::TextPolygon, bounds=%d,%d,%d,%d\n",
	//	bounds.x(), bounds.y(), bounds.width(), bounds.height() );
	
	// Get the points of the string into vvv
	//	
	if (text.isNull())
		text = QString("text");
	string ss = string(text.latin1());
	//SHOW2(ss.size(), ss);
#ifdef USE_CHARTABLE2
    vector<Point> flatv;
	vector< vector<Point> > vv;
	Chartable2 *ct = &chartable2;
	SHOW(ct);
	SHOW(vv.size());
	
	Chartable2::StringPath str_path(ct, ss, 64.0, -64.0);

	uint n = str_path.nFillablePaths();
	for (uint i=0; i<n; i++) {
		const Chartable2::path_t & v = str_path.fillablePath(i);
		for (uint j=0; j<v.size(); j++) {
			flatv.push_back(v[j]);
			SHOW(v[j]);	
		}
	}
#else
	vector<vector< vector<Point> > > vvv;
	chartable.getPoints( ss , 0.0, 0.0, 64.0, -64.0, vvv);
	
	// copy all the points into a one-dimensional vector,
	// count the points for each path,
	// count the points for each letter,
	// and count the number of paths.
	
	totalpaths = 0;
	
	vector<Point> flatv;
	for (unsigned int i=0; i < vvv.size(); i++) {
		
		totalpaths += vvv[i].size();
		int n = 0;
		for (unsigned int j=0; j < vvv[i].size(); j++)  {
			int pointsThisPath = vvv[i][j].size();
			for (int k=0; k < pointsThisPath; k++)  {
				flatv.push_back( vvv[i][j][k] );
				SHOW(vvv[i][j][k]);
			}
			pointsPerPath.push_back(pointsThisPath);
			n += pointsThisPath;
		}
		pointsPerLetter.push_back(n);
	}
	assert((uint)totalpaths == pointsPerPath.size());
#endif
	
	// create the outline
	//
	Outline * ol = new Outline(flatv);
	//SHOW2(flatv.size(), ol->size() );
	ol->toRect(bounds.x(),
	           bounds.y(),
	           bounds.width(),
	           bounds.height(), true, 1, 1);

	setOutline(ol);

	// save the paths
	//
	SHOW0 << "===========\n";
	
#ifdef USE_CHARTABLE2
	
	// create the `fillables' vector, which holds QPointArrays
	// that are to be drawn as filled polygons.
	//
	uint nfill = str_path.nFillablePaths();
	for (uint i=0; i<nfill; i++)
	{
		const Chartable2::path_t & ipath = str_path.fillablePath(i);
		uint npoint = ipath.size();
		fillables.push_back( QPointArray(npoint) );
		QPointArray & pa = fillables[i];
		
		SHOW(npoint);
		for (uint j=0; j<npoint; j++)
		{
			Point p = ol->transformPoint( ipath[j] );
			QPoint qq = QPoint( qRound(p.x()), qRound(p.y()) );
			SHOW2(qq.x(), qq.y());
			pa[j] = qq;
		}
	}

	// Create the `lines' vector, which holds QPointArrays
	// that are to be drawn as lines.
	//
	uint nlines = str_path.nLinePaths();
	for (uint i=0; i<nlines; i++)
	{
		const Chartable2::path_t & pth = str_path.linePath(i);
		uint npoint = pth.size();
		lines.push_back( QPointArray(npoint) );
		QPointArray & line_pa = lines[i];
		for (uint j=0; j < npoint; j++ ) {
			Point p = ol->transformPoint( pth[j] );
			QPoint lq = QPoint( qRound(p.x()), qRound(p.y()) );
			SHOW2(lq.x(), lq.y());
			line_pa[j] = lq;
		}
	}

#else	
	int pathIndex = 0;
	//SHOW(vvv.size());
	// for each letter ...
	for (unsigned int i=0; i < vvv.size(); i++)
	{
		QPointArray letter;
		
		// for each path of this letter ...
		for (unsigned int j=0; j < vvv[i].size(); j++)
		{
		    QPointArray pa( pointsPerPath[pathIndex] + 1 );
			int jj=0;
			SHOW0 << "letter " << i << ", path " << j << ":" << endl;
		    // for each point of this path ...
		    for (unsigned int k=0; k < vvv[i][j].size(); k++)
		    {
		    	Point p = ol->transformPoint( vvv[i][j][k] );
				pa[jj++] = QPoint( qRound(p.x()), qRound(p.y()) );
			    SHOW2(pa[jj-1].x(), pa[jj-1].y());
			}
			if (jj > 0) {
				pa[jj++] = pa[0];  // close the path
				SHOW2(pa[jj-1].x(), pa[jj-1].y());
		    	paths.push_back(pa);
		    	pathIndex++;
		    	letter.putPoints(letter.size(), pa.size(), pa);
		  	}
		}
		letters.push_back(letter);
	}
#endif
	SHOW0 << "===========\n";
}


TextPolygon::~TextPolygon()
{
}

void TextPolygon::drawShape(QPainter & p)
{
	SHOW0 << "--enter TextPolygon::drawShape\n";
	p.save();
	
#if 0
	// draw some text using drawText(), for comparison
	//
	p.save();
	p.translate( 0, qRound(canvas()->height()*0.9) );
	p.scale(2.4,2.4);
	QString newText = text;
	newText += QString(" - using drawText");
	//QFont f = p.font();
	//f.setStyleStrategy(QFont::PreferAntialias);
	//p.setFont(f);
	p.drawText( QPoint(0,0), newText );
	p.restore();
#endif

	p.translate(x(),y());
	
#ifdef USE_CHARTABLE2
	QPen pen_save = p.pen();
	p.setPen( NoPen );
	for (unsigned int i=0; i<fillables.size(); ++i) {
		p.drawPolygon(fillables[i]);
		//break;
	}
	p.setPen(pen_save);
	if (p.pen() == NoPen) {
		p.setPen( p.brush().color() );
	}
	SHOW(lines.size());
	for (unsigned int i=0; i<lines.size(); ++i) {
		p.drawPolyline(lines[i]);
		//break;
	}
#else	
	for (unsigned int i=0; i<paths.size(); ++i) {
		p.drawPolyline(paths[i]);
		//break;
	}
	
	p.setPen(NoPen);
	SHOW(letters.size());
	for (unsigned int i=0; i<letters.size(); ++i) {
		p.drawPolygon(letters[i]);
		//break;
	}
#endif
	
	p.restore();
	SHOW0 << "--exit TextPolygon::drawShape\n";
}



