/* Copyright 2002 Scott Haywood

This file is part of the Label program.

Label is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Label is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Label; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef LABEL_TEXT_H
#define LABEL_TEXT_H

/****************************************************************
**
**	TextPolygon is a QCanvasPolygon constructed
**	from a text string.  Unlike QCanvasText, it
**	provides access to the shape of	the string.
**
**  This class uses a built-in font.
**
****************************************************************/

#include <iostream>
#include <vector>

#include <qcanvas.h>
#include <qpainter.h>

#include "outline.h"
#include "point.h"
#include "outlined_polygon.h"

/*
 */
class TextPolygon : public OutlinedPolygon
{
public:
	TextPolygon(QCanvas *cp, const QString & s, const QRect & bounds);
	~TextPolygon();

	// Reimmplements QCanvasPolygonal
	void drawShape(QPainter & p);
	
	// Gets the text as a QString
	QString getText() const { return text; }

private:
	// don't allow copy constructor or assignment
	// This makes it easier to manage pointers
	// that the destructor deletes.
	//
	TextPolygon(const TextPolygon &t) : OutlinedPolygon(t.canvas())
		{ throw "TextPolygon: no copy constructor\n"; }
	TextPolygon& operator=(const TextPolygon &)
		{ throw "TextPolygon: no assignment operator\n"; }
	
	QString text;                  // the string
#define USE_CHARTABLE2
#ifdef  USE_CHARTABLE2
	vector< QPointArray > fillables;
	vector< QPointArray > lines;
#else
	vector< QPointArray > letters;  // the shape of each letter
	vector<int> pointsPerLetter;  // number of points in a letter
	vector< QPointArray > paths;  // the shape of each path
	vector<int> pointsPerPath;	// number of points in each path.
	int totalpaths; // total number of paths
#endif
};

#endif LABEL_TEXT_H
