/* Copyright 2002 Scott Haywood

This file is part of the Label program.

Label is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Label is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Label; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/****************************************************************
**
** Implementation of XMark, a QCanvasItem that draws an X
**
****************************************************************/

#include "xmark.h"

XMark::XMark(QCanvas *cp, const QPoint & p)
	: QCanvasPolygonalItem(cp),
	  pa(4)
{
	move(p.x(), p.y());
}

XMark::XMark(QCanvas *cp, double x, double y)
	: QCanvasPolygonalItem(cp),
	  pa(4)
{
	move(x, y);
}


// copy constructor, not intended to be used
XMark::XMark(const XMark &)
	 : QCanvasPolygonalItem(canvas())
		{ throw "XMark copy constructor not implemented"; }


void XMark::setpa () const // "const" since pa is mutable
{
	int a = qRound(x());
	int b = qRound(y());
	pa.setPoints(4, a-6, b-6,
	                a+6, b-6,
	                a+6, b+6,
	                a-6, b+6);
}

QPointArray XMark::areaPoints () const
{
	setpa();
	return pa;
}

void XMark::drawShape(QPainter & p)
{
	int a = qRound(x());
	int b = qRound(y());
	p.drawLine( a-5, b-5,
	            a+5, b+5);
	p.drawLine( a+5, b-5,
	            a-5, b+5);
}

